"""uilayer

python user interface layer to monitor progress of long-running
algorithms
"""
import os
import sys
import warnings

__all__ = ["dummy", "ui", "stdout"]

try:
    import wx
except:
    pass
else:
    __all__ += ["wxdlg"]


class base_ui():
    """
        This is the main class from which all interfaces are derived.
        It is quiet, meaning it does not output any progress data, but
        it will display warings and errors in the console.
    """

    def __init__(self, steps=None, pids=None, progress=None,
                 subpids=None, show_warnings=True, totalsteps=None,
                 dirname=None, **kwargs):
        """
            Creates an instance and sets initial parameters for progress
            dialogs (optional).


            Parameters:
            -----------
            pids, steps, progress, subpids : optional
                See documentation of self.SetProgress
            show_warnings : bool
                Display warnings of/in the interface.
            totalsteps : int or None
                Known number of total steps of all progresses. This
                number can be larger than the sum of all steps,
                inferring knowledge on the progresses to come.


            See Also
            --------
            SetProgress : Sets initial parameters manually
        """
        if dirname is None:
            self._dirname = os.path.expanduser("~")
        self._abortlist = list()
        self._totalsteps = totalsteps
        self._currentstep = 0
        self._single_progress = False
        self.ShowWarnings(show_warnings)
        self.SetProgress(steps=steps,
                         pids=pids,
                         progress=progress,
                         subpids=subpids)

    def GetDir(self):
        """ Returns the current working directory.
        """
        return self._dirname

    def Iterate(self, pid=None, subpid=0, **kwargs):
        """ Iterate progress.

        Increments the "current step" of a progress by one.
        Warns the user if the step is above "total steps".

        Also looks for the pid in self._abortlist and returns it to
        the process to stop it.
        """
        if self._totalsteps is not None:
            self._currentstep += 1

        if pid is None:
            if self._single_progress == False:
                self.warn("I do not know which progress to iterate." +
                          " Please specify the kwarg pid.")
            pid = 0

        pid = int(pid)
        subpid = int(subpid)

        for i in range(len(self._pids)):
            if self._pids[i] == pid and self._subpids[i] == subpid:
                self._progress[i] += 1

        self.Update()

        if pid in self._abortlist:
            # Try to stop the process if it supports that.
            # Returning anything else but None will stop the process.
            return pid

    def Finalize(self, pid=None, subpid=0, **kwargs):
        """ Triggers output that a process is finished
        """
        pass

    def SelectPath(self, **kwargs):
        """ Select a file or folder from the file system.

        This does not do anything here.
        """
        pass

    def SetDir(self, dirname):
        """ Sets the current working directory.
        """
        self._dirname = dirname

    def SetProgress(self, steps=None, pids=None, progress=None,
                    subpids=None, **kwargs):
        """
            Set the initial parameters of the processes that are to be
            monitored by this class.


            Parameters
            ----------
            pids : Array-like dtype(int) or int, length N
                Process identifiers
            steps : Array-like dtype(int) or int, length N
                Maximum number of steps for each process
            progress : Array-like dtype(int) or int, length N
                Current step in the progress counting up until `steps`
            subpids : Array-like dtype(int) or int, length N
                Sub-process identifiers - useful for multithreading

            The following kwargs combinations are possible:
                1. pids, steps, progress, subpids all `None`
                    Nothing will happen
                2. pids, steps, progress, subpids all some int
                    Only one process of unknown length
                3. pids, steps, progress, subpids all integer `list`s
                    Multiple processes are assumed
                4. pids, progress, subpids all `None` and steps int
                    Only one process of known length
                5. pids, progress, subpids all `None` and steps `list`
                    Multiple processes of known length - internally
                    the processes get `pids` that are enumerated from
                    zero onwards and `progress` is set no zeros.

            Returns
            -------
            success : bool
                True if data was set and False if nothing was set.
        """
        self._single_progress = False
        self._no_data = False
        self._known_length = False
        if (pids is None and steps is None and progress is None):
            # 1. Nothing will happen
            self._no_data = True
            return False
        elif (pids is not None and steps is not None and
              progress is not None):
            # 2. Only one process
            # or
            # 3. Multiple processes are assumed
            self._known_length = False
        elif (pids is None and steps is not None and progress is None):
            self._known_length = True
            if isinstance(steps, (int, float, complex)):
                # 4. Only one process of known length
                self._single_progress = True
                pids = 0
                progress = 0
            else:
                # 5. Multiple processes of known length - internally
                #    the processes get `pids` that are enumerated from
                #    zero onwards and `progress` is set no zeros.
                pids = range(len(steps))
                progress = [0] * len(steps)
        else:
            raise NameError("Your set of parameters is invalid.")

        # Convert input to list
        if isinstance(pids, (int, float, complex)):
            # Only one progress
            self._single_progress = True
            pids = [pids]
        if isinstance(steps, (int, float, complex)):
            steps = [steps]
        if isinstance(progress, (int, float, complex)):
            progress = [progress]
        if isinstance(subpids, (int, float, complex)):
            subpids = [subpids]

        self._pids = pids
        # Defines how many steps should be used to split the total
        # computation. The function self.progress will be used to
        # follow the steps of the algorithm.
        self._steps = steps
        self._progress = progress

        if subpids is None:
            self._subpids = [0] * len(self._pids)
        else:
            self._subpids = subpids

        return True

    def ShowWarnings(self, show_warnings):
        """
            show_warnings : bool
                enable or disable warning
        """
        self._show_warnings = show_warnings

    def Update(self, **kwargs):
        """  Read and write from and to the user interface

        This does not do anything here.
        """
        pass

    def warn(self, msg):
        """ Warns the user when there is a problem. """
        if self._show_warnings:
            warnings.warn(msg)


class dummy(object):
    """
        This is a dummy class that can be used to improve speed.
        There will be no output.
    """

    def __init__(self, *args, **kwargs):
        pass

    def Finalize(self, *args, **kwargs):
        pass

    def Iterate(self, *args, **kwargs):
        pass

    def SelectPath(self, *args, **kwargs):
        pass

    def SetDir(self, *args, **kwargs):
        pass

    def SetProgress(self, *args, **kwargs):
        pass

    def ShowWarnings(self, *args, **kwargs):
        pass

    def Update(self, *args, **kwargs):
        pass

    def warn(self, *args, **kwargs):
        pass


class stdout(base_ui):
    """ Writes progress into standard output.

    """

    def __init__(self, **kwargs):
        """ Stuff that needs to be done in the beginning.

        """
        base_ui.__init__(self, **kwargs)
        self._maxlength = get_terminal_width()

    def Finalize(self, pid=None, subpid=0, name=None, **kwargs):
        """ Triggers output when a process is finished.

        """
        if pid is None:
            self.warn("I do not know which progress to finalize." +
                      " Please specify the kwarg pid.")

        pid = int(pid)
        subpid = int(subpid)

        msg = "Progress finished: {}-{}".format(pid, subpid)
        if name is not None:
            msg += " ({})".format(name)
        dn = max(self._maxlength - len(msg), 0)

        sys.stdout.write("\r{}{}\n".format(msg, dn * " "))
        sys.stdout.flush()

    def Update(self, **kwargs):
        """ Read and write from and to the user interface.

        The algorithm wants to tell the user something.
        We use the last line of the standard output to write the
        progress of all algorithms.
        """
        startline = u"Progress: "
        msg = startline
        if self._single_progress:
            if self._steps[0] == 0:
                perc = int(self._progress[0])
                msg += u"{}".format(perc)
            else:
                perc = self._progress[0] / self._steps[0] * 100.
                msg += u"{:0.2f}%".format(perc)
        else:
            for i in range(len(self._pids)):
                # compute percentage
                if self._pids[i] != -1:
                    if self._steps[i] == 0:
                        perc = int(self._progress[i])
                        msg += u"{}-{}@{}|".format(
                            self._pids[i], self._subpids[i], perc)
                    else:
                        perc = self._progress[i] / self._steps[i] * 100.
                        msg += u"{}-{}@{:0.2f}%|".format(
                            self._pids[i], self._subpids[i], perc)
        msg = msg.strip(" |")

        dn = max(self._maxlength - len(msg), 0)

        newline = "\r{}{}".format(msg, dn * " ")

        if len(newline) > self._maxlength + 1:
            warnings.warn("Terminal width too small. Output cropped.")
            newline = newline[:self._maxlength - 1]

        if newline.strip() == startline.strip():
            newline = self._maxlength * " " + "\r"

        sys.stdout.write(newline)
        sys.stdout.flush()


class wxdlg(base_ui):
    """ Displays propgress in a wx dialog window.

    Can be used from within a wx.App.
    """

    def __init__(self, parent=None, title="Progress", **kwargs):
        """ Stuff that needs to be done in the beginning.

        """
        self.parent = parent
        self.title = title
        base_ui.__init__(self, **kwargs)

    def Destroy(self):
        """ Destroys the wxdlg class.
        """
        try:
            self.Finalize()
        except:
            pass

    def Finalize(self, pid=None, subpid=0, name=None, **kwargs):
        """ Triggers output when a process is finished.

        """
        if self._single_progress:
            self.dlg.Destroy()
        elif (self._totalsteps is not None and
              self._totalsteps <= self._currentstep):
            self.dlg.Destroy()
        else:
            self.Update()

    def SelectPath(self, dirname=None, title="Open file",
                   wildcards=["*"], fname="All files", mode="r",
                   **kwargs):
        """ Select a file or folder from the file system.

        Opens a wx.FileDialog for selecting files.

        Parameters
        ----------
        mode : str
            File mode operations read "r" or write "w" or
            read multiple files "mr".

        Returns the files in a list.
        """
        if dirname is None:
            dirname = self._dirname
        front = " ("
        rear = ")|"
        for wc in wildcards:
            front += "*.{}, ".format(wc)
            rear += ";*{}".format(wc)
        front.strip(", ")
        wcstring = fname + front + rear

        if mode == "r":
            wxmode = wx.FD_OPEN
        else:
            raise NotImplementedError(
                "Mode '{}' not supported".format(mode))
        dlg = wx.FileDialog(self.parent, title, dirname, "",
                            wcstring, wxmode)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self._dirname = os.path.split(path)[0]
            dlg.Destroy()
            return path
        else:
            return

    def SetProgress(self, **kwargs):
        """ Sets progress data from base_ui and dialog parameters

        """
        if base_ui.SetProgress(self, **kwargs):

            style = wx.PD_SMOOTH | wx.PD_AUTO_HIDE | wx.PD_CAN_ABORT

            if self._known_length:
                style |= wx.PD_REMAINING_TIME

            if self._totalsteps is not None:
                maxsteps = self._totalsteps
            else:
                maxsteps = 0
                for i in self._steps:
                    maxsteps += i

            self._total_counter = 0

            # known length?
            title = self.title
            if not self._single_progress:
                title += " ({} processes)".format(len(self._steps))

            self.dlg = wx.ProgressDialog(
                title=title,
                message="Running : ",
                maximum=maxsteps,
                parent=self.parent,
                style=style)

            self.dlg.SetMinSize((300 + 20 * len(self._steps), 80))

    def Update(self, **kwargs):
        """ Read and write from and to the user interface.

        Checks if the user has pressed 'Cancel' and updates the
        Progress bar.
        """
        self._total_counter += 1

        msg = "Running : "

        if self._single_progress:
            if self._known_length:
                msg += "PID {}-{}".format(self._pids[0],
                                          self._subpids[0])
            else:
                msg += "PID {}-{}@{}".format(self._pids[0],
                                             self._subpids[0],
                                             self._progress[0])
        else:
            for i in range(len(self._pids)):
                if self._pids[i] != -1:
                    if self._known_length:
                        msg += u"PID {}-{}@{}, ".format(
                            self._pids[i], self._subpids[i], self._progress[i])
                    else:
                        msg += u"PID {}-{}, ".format(
                            self._pids[i], self._subpids[i])

        msg = msg.strip(" ,")

        if self._known_length:
            cont = self.dlg.Update(self._total_counter, msg)[0]
        else:
            cont = self.dlg.UpdatePulse(newmsg=msg)[0]

        if cont is False:
            # On next iteration, algorithms can abort if they
            # support it.
            self._abortlist += self._pids


def get_terminal_size(fd=1):
    """
    Returns height and width of current terminal. First tries to get
    size via termios.TIOCGWINSZ, then from environment. Defaults to 25
    lines x 80 columns if both methods fail.

    :param fd: file descriptor (default: 1=stdout)

    Author: cas (http://blog.taz.net.au/author/cas/)
    """
    try:
        import fcntl
        import termios
        import struct
        hw = struct.unpack('hh', fcntl.ioctl(fd, termios.TIOCGWINSZ, '1234'))
    except:
        try:
            hw = (os.environ['LINES'], os.environ['COLUMNS'])
        except:
            hw = (25, 80)

    return hw


def get_terminal_height(fd=1):
    """
    Returns height of terminal if it is a tty, 999 otherwise

    :param fd: file descriptor (default: 1=stdout)
    """
    if os.isatty(fd):
        height = get_terminal_size(fd)[0]
    else:
        height = 999

    return height


def get_terminal_width(fd=1):
    """
    Returns width of terminal if it is a tty, 999 otherwise

    :param fd: file descriptor (default: 1=stdout)
    """
    if os.isatty(fd):
        width = get_terminal_size(fd)[1]
    else:
        width = 999

    return width



import numpy as np
from scipy import optimize as spopt


def fit_exp(times, trace):
    """Fit an exponential function to the given trace.


    Parameters
    ----------
    times : ndarray of length N
        x-values

    trace : ndarray of length N
        y-values


    Returns
    -------
        parameters, function
    """
    # Set starting parameters for exponential fit
    def expfunc(p, x): return p[0] * np.exp(-x / p[1])
    # parms = [ampl, decaytime]
    parms = np.zeros(2, dtype=np.float32)
    ltr = len(trace)
    border = np.int(np.ceil(ltr / 50.))
    parms[0] = 1. * trace[:border].mean()
    parms[1] = times[-1] / np.log(parms[0])
    # Fit function is an exponential
    # Function to minimize via least squares
    # f_min = lambda p, x: expfunc(np.abs(p), x) - trace

    def f_min(p, x):
        p = np.abs(p)
        return expfunc(np.abs(p), x) - trace
    # Least squares
    popt, chi = spopt.leastsq(f_min, parms[:], args=(times))
    return np.abs(popt), expfunc


def fit_gauss(amplitudes, frequencies,  argmax):
    # Set starting parameters for gaussian fit
    # parms = [freq, ampl, sigma]
    parms = np.zeros(3, dtype=np.float32)
    parms[0] = frequencies[argmax]
    parms[1] = amplitudes[argmax]
    parms[2] = abs(frequencies[1] - frequencies[2]) * 2
    # Fit function is a gaussian

    def gauss(p, x):
        expnt = np.exp(-((x - p[0]) / p[2]) ** 2 / 2)
        norm = p[1] / (p[2] * np.sqrt(2 * np.pi))
        return expnt * norm

    # Function to minimize via least squares
    def f_min(p, x):
        return gauss(p, x) - amplitudes
    # Least squares
    popt, chi = spopt.leastsq(f_min, parms[:], args=(frequencies))
    return np.abs(popt), gauss



import numpy as np

from pyscanfcs import bin_pe


def test_bin_photon_events():
    data = np.array([5,  # 5
                     1, 1, 1,  # 8
                     4, 1, 1, 1,  # 15
                     1, 1, 1,  # 18
                     # blank
                     8, 1, 1,  # 28
                     ], dtype=np.uint32)
    binf = bin_pe.bin_photon_events(data=data, t_bin=5.0001)
    binned = np.fromfile(binf, dtype="uint16", count=-1)
    assert np.all(binned == np.array([1, 3, 4, 3, 0, 3]))


if __name__ == "__main__":
    # Run all tests
    loc = locals()
    for key in list(loc.keys()):
        if key.startswith("test_") and hasattr(loc[key], "__call__"):
            loc[key]()



"""filetype definitions"""
import astropy.io.fits
import numpy as np
from skimage.external import tifffile


def openAny(path, callback=None):
    """load any supported file type"""
    methods = methods_binned.copy()
    methods.update(methods_stream)

    for key in list(methods.keys()):
        if path.endswith(key):
            return methods[key](path, callback)


def openDAT(path, callback=None, cb_kwargs={}):
    """Load "Flex02-12D" correlator.com files

    We open a .dat file as produced by the "Flex02-12D" correlator in photon
    history recorder mode.
    The file contains the time differences between single photon events.

    Parameters
    ----------
    path : str
        Path to file
    callback : callable or None
        Callback function to be called throughout the algorithm. If the
        return value of `callback` is not None, the function will abort.
        Number of function calls: 3
    cb_kwargs : dict, optional
        Keyword arguments for `callback` (e.g. "pid" of process).

    Returns
    -------
    info: dict
        Dictionary containing the "system_clock" in MHz and the
        "data_stream" (photon arrival time event stream).
        Returns `None` if the progress was aborted through the
        callback function.


    Notes
    -----
    Raw data file format (taken from manual):
     1. The file records the difference in system clock ticks (1/60 us)
        between photon event.
     2. The first byte identifies the format of the file 8 : 8 bit, 16: 16 bit
     3. The second byte identifies the system clock. 60MHz.
     4. The time unit is 1/system clock.
     5. 16 bit format. Each WORD (2 bytes) represents a photon event,
        time = WORD/system clock, unless the value is 0xFFFF, in which case,
        the following four bytes represent a photon event.
     6. 8 bit format: Each BYTE represents a photon event unless the value is
        0xFF, in which case, the BYTE means 255 clock ticks passed without a
        photon event. For example 0A 0B FF 08 means there are three
        photon events. The time series are 0x0A+1, 0x0B+1, 0xFF+8+1.

    """
    # open file
    filed = open(path, 'rb')
    # 1st byte: get file format
    # should be 16 - for 16 bit
    fformat = int(np.fromfile(filed, dtype="<u1", count=1))
    # 2nd byte: read system clock
    system_clock = int(np.fromfile(filed, dtype="<u1", count=1))
    if fformat == 8:
        # No 8 bit format supported
        raise ValueError("8 bit format not supported!")
    elif fformat == 32:
        # (There is an utility to convert data to 32bit)
        data = np.fromfile(filed, dtype="<u4", count=-1)
    elif fformat == 16:
        # convert 16bit to 32bit
        # Read the rest of the file in 16 bit format.
        # Load bunch of Data
        data16 = np.fromfile(filed, dtype="<u2", count=-1)
        # Now we need to check if there are any 0xFFFF values which would
        # mean, that we do not yet have the true data in our array.
        # There is 32 bit data after a 0xFFFF = 65535
        if callback is not None:
            ret = callback(**cb_kwargs)
            if ret is not None:
                return

        # occurences of large values
        occ = np.where(data16 == 65535)[0]
        N = len(occ)

        if callback is not None:
            ret = callback(**cb_kwargs)
            if ret is not None:
                return

        # Make a 32 bit array
        data = np.uint32(data16)
        data[occ] = data16[occ + 1] + data16[occ + 2] * 65536

        if callback is not None:
            ret = callback(**cb_kwargs)
            if ret is not None:
                return None

        # Now delete the zeros
        zeroids = np.zeros(N * 2, dtype=int)
        zeroids[::2] = occ + 1
        zeroids[1::2] = occ + 2
        data = np.delete(data, zeroids)
    else:
        raise ValueError("Unknown format: {} bit".format(fformat))
    filed.close()

    info = {"data_stream": data,
            "system_clock": system_clock
            }

    return info


def openFITS(fname, callback=None):
    """ load .fits files """
    info = dict()

    fits = astropy.io.fits.open(fname)
    series = fits[0]
    head = series.header

    info["type"] = "binned"
    info["data_binned"] = series.data
    info["system_clock"] = head['SysClck']

    try:
        info["total_time"] = head['Total']
    except KeyError:
        info["total_time"] = None

    try:
        info["line_time"] = head['Tline']
    except KeyError:
        # Old "line scanning time" has new name: "scan cycle time"
        info["line_time"] = head['Tcycle']

    info["bin_time"] = head['Tbin']

    try:
        info["bin_shift"] = head['Binshift']
    except KeyError:
        info["bin_shift"] = None

    info["bins_per_line"] = len(series.data[0])

    info["size"] = series.data.shape[0] * series.data.shape[1]

    return info


def openLSM(fname, callback=None):
    """ open LSM780 file using tifffile """
    info = dict()
    info["type"] = "binned"

    lsm = tifffile.TiffFile(fname)

    # two channels 0, 1?
    info["data_binned"] = lsm.asarray()[0]
    info["system_clock"] = 1

    page = lsm.pages[0]
    # pixel time in us
    info["bin_time"] = page.cz_lsm_scan_information["tracks"][0]["pixel_time"]

    # line time in s -> us
    info["line_time"] = (page.cz_lsm_time_stamps[1] -
                         page.cz_lsm_time_stamps[0]) * 1e6

    # total time in s -> us
    info["total_time"] = (page.cz_lsm_time_stamps[-1] -
                          page.cz_lsm_time_stamps[0]) * 1e6

    info["bins_per_line"] = info["data_binned"].shape[1]
    info["size"] = info["data_binned"].shape[0] * info["data_binned"].shape[1]
    info["bin_shift"] = None

    return info


methods_stream = {"dat": openDAT}

methods_binned = {"fits": openFITS,
                  "lsm": openLSM}
#                  "tif": openTIF}

wx_dlg_wc_stream = "stream format ("
wx_dlg_wc_stream_end = ")|"
for key in list(methods_stream.keys()):
    wx_dlg_wc_stream += "*.{}, ".format(key)
    wx_dlg_wc_stream_end += "*.{};".format(key)
wx_dlg_wc_stream = wx_dlg_wc_stream.strip(
    " ,") + wx_dlg_wc_stream_end.strip(";")


wx_dlg_wc_binned = "binned format ("
wx_dlg_wc_binned_end = ")|"
for key in list(methods_binned.keys()):
    wx_dlg_wc_binned += "*.{}, ".format(key)
    wx_dlg_wc_binned_end += "*.{};".format(key)
wx_dlg_wc_binned = wx_dlg_wc_binned.strip(
    " ,") + wx_dlg_wc_binned_end.strip(";")



"""Miscellaneous classes"""
import codecs
import os
import sys

import numpy as np
import wx
import wx.lib.delayedresult as delayedresult


# The icon file was created with
# img2py -i -n Main PyScanFCS_icon.png icon.py
from . import icon
from . import edclasses


class ArtificialDataDlg(wx.Frame):
    # This tool is derived from a wx.frame.
    def __init__(self, parent):
        self.parent = parent
        # Get the window positioning correctly
        pos = self.parent.GetPosition()
        pos = (pos[0] + 100, pos[1] + 100)
        wx.Frame.__init__(self, parent=self.parent,
                          title="Create artificial test data", pos=pos,
                          style=wx.DEFAULT_FRAME_STYLE | wx.FRAME_FLOAT_ON_PARENT)

        # Content
        self.panel = wx.Panel(self)

        topSizer = wx.BoxSizer(wx.VERTICAL)

        topSizer.Add(wx.StaticText(self.panel,
                                   label="Create artificial test data\n" +
                                   "with exponentially correlated noise.\n"))

        colsizer = wx.FlexGridSizer(4, 2, 20)

        # decay time
        colsizer.Add(wx.StaticText(self.panel,
                                   label="Correlation time [ms]: "))
        self.WXDecay = edclasses.FloatSpin(self.panel, value="100")
        colsizer.Add(self.WXDecay)

        # line scanning time
        colsizer.Add(wx.StaticText(self.panel,
                                   label="Scan cycle time [ms]: "))
        self.WXLine = edclasses.FloatSpin(self.panel, value="0.714")
        colsizer.Add(self.WXLine)

        # number of samples
        colsizer.Add(wx.StaticText(self.panel,
                                   label="Number of samples: "))
        self.WXNumber = wx.SpinCtrl(self.panel, -1, min=10000,
                                    max=500000000, value="500000")
        colsizer.Add(self.WXNumber)

        # data format
        colsizer.Add(wx.StaticText(self.panel, label="Data format: "))
        self.WXDropInt = wx.ComboBox(self.panel, -1, "uint16", (15, 30),
                                     wx.DefaultSize, ["uint16", "uint32"],
                                     wx.CB_DROPDOWN | wx.CB_READONLY)
        colsizer.Add(self.WXDropInt)

        topSizer.Add(colsizer)

        btndo = wx.Button(self.panel, wx.ID_CLOSE, 'Create .dat file')
        # Binds the button to the function - close the tool

        self.Bind(wx.EVT_BUTTON, self.OnCreateFile, btndo)

        topSizer.Add(btndo)
        self.panel.SetSizer(topSizer)
        topSizer.Layout()
        topSizer.Fit(self)
        # self.SetMinSize(colsizer.GetMinSizeTuple())
        # Icon
        if parent.MainIcon is not None:
            wx.Frame.SetIcon(self, parent.MainIcon)
        self.Show(True)

    def OnClose(self, event=None):
        # This is a necessary function for PyCorrFit.
        # Do not change it.
        self.parent.toolmenu.Check(self.MyID, False)
        self.parent.ToolsOpen.__delitem__(self.MyID)
        self.Destroy()

    def OnCreateFile(self, e=None):
        linetime = self.WXLine.GetValue()
        taudiff = self.WXDecay.GetValue()
        N = self.WXNumber.GetValue()
        inttype = self.WXDropInt.GetValue()

        if inttype == "uint16":
            dtype = np.uint16
        else:
            dtype = np.uint32

        # file dialog
        filename = "test_tau{}ms_line{}ms_N{}_{}.dat".format(
            taudiff, linetime, N, inttype)

        dlg = wx.FileDialog(self, "Save as .dat file", "./", filename,
                            "DAT files (*.dat)|*.dat;*.daT;*.dAt;*.dAT;*.Dat;*.DaT;*.DAt;*.DAT",
                            wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        # user cannot do anything until he clicks "OK"
        if dlg.ShowModal() == wx.ID_OK:
            # Workaround for Ubuntu 12.10 since 0.2.0
            path = dlg.GetPath()
        else:
            return

        wx.BeginBusyCursor()

        # start new thread
        args = N, taudiff, linetime, dtype, path
        delayedresult.startWorker(_CreateConsumer, _CreateWorker,
                                  wargs=(args,),)


def GenerateExpNoise(N, taud=20., variance=1., deltat=1.):
    """
        Generate exponentially correlated noise.
    """
    # length of mean0 trace
    N_steps = N
    dt = deltat
    # AR-1 processes - what does that mean?
    # time constant (inverse of correlationtime taud)
    g = 1. / taud
    # variance
    s0 = variance

    # normalization factor (memory of the trace)
    exp_g = np.exp(-g * dt)
    one_exp_g = 1 - exp_g
    z_norm_factor = np.sqrt(1 - np.exp(-2 * g * dt)) / one_exp_g

    # create random number array
    # generates random numbers in interval [0,1)
    randarray = np.random.random(N_steps)
    # make numbers random in interval [-1,1)
    randarray = 2 * (randarray - 0.5)

    # simulate exponential random behavior
    z = np.zeros(N_steps)
    z[0] = one_exp_g * randarray[0]
    for i in np.arange(N_steps - 1) + 1:
        z[i] = exp_g * z[i - 1] + one_exp_g * randarray[i]

    z = z * z_norm_factor * s0
    return z


def MakeDat(linetime, noisearray, dtype, filename):
    """ Create a .dat file (like Photon.exe).
        System clock is fixed to 60MHz.
        linetime [s]
        noisearray integer array (uint16 or uint32)
    """
    NewFile = open(filename, 'wb')
    if dtype == np.uint32:
        newformat = np.uint8(32)
    elif dtype == np.uint16:
        newformat = np.uint8(16)
    else:
        raise ValueError
    newclock = np.uint8(60)
    NewFile.write(newformat)
    NewFile.write(newclock)
    noisearray = dtype(noisearray)
    # Create matrix. Each line is a scan.
    timeticks = linetime * newclock * 1e6  # 60MHz
    half1 = np.ceil(timeticks / 2)
    half2 = np.floor(timeticks / 2)
    for i in np.arange(len(noisearray)):
        # Create a line
        N = noisearray[i]
        if N == 0:
            line = np.zeros(1, dtype=dtype)
            # Only one event at the far corner
            line[0] = timeticks
            line.tofile(NewFile)
        else:
            line = np.ones(N + 1, dtype=dtype)
            # events are included between two far events
            line[0] = half1 - len(line)
            line[-1] = half2
            line.tofile(NewFile)
    NewFile.close()


def removewrongUTF8(name):
    newname = u""
    for char in name:
        try:
            codecs.decode(char, "UTF-8")
        except:
            pass
        else:
            newname += char
    return newname


def getMainIcon(pxlength=32):
    """ *pxlength* is the side length in pixels of the icon """
    # Set window icon
    iconBMP = icon.getMainBitmap()
    # scale
    image = wx.Bitmap.ConvertToImage(iconBMP)
    image = image.Scale(pxlength, pxlength, wx.IMAGE_QUALITY_HIGH)
    iconBMP = wx.Bitmap(image)
    iconICO = wx.Icon(iconBMP)
    return iconICO


def findprogram(program):
    """ Uses the systems PATH variable find executables"""
    path = os.environ['PATH']
    paths = path.split(os.pathsep)
    for d in paths:
        if os.path.isdir(d):
            fullpath = os.path.join(d, program)
            if sys.platform[:3] == 'win':
                for ext in '.exe', '.bat':
                    program_path = fullpath + ext
                    if os.path.isfile(fullpath + ext):
                        return (1, program_path)
            else:
                if os.path.isfile(fullpath):
                    return (1, fullpath)
    return (0, None)


def _CreateConsumer(e=None):
    wx.EndBusyCursor()


def _CreateWorker(args):
    N, taudiff, linetime, dtype, path = args

    # deltat is linetime
    noisearray = GenerateExpNoise(N, taud=taudiff, deltat=linetime)
    noisearray -= np.min(noisearray)
    noisearray *= 30. / np.max(noisearray)
    #noisearray += 5
    noisearray = np.uint32(noisearray)

    # Create 32bit and 16bit binary .dat files
    # translate linetime in bins of taud
    ltbin = linetime / 1000
    MakeDat(ltbin, noisearray, dtype, path)



"""PyScanFCS loader"""
from os.path import dirname, abspath
import warnings

import sys
sys.path.insert(0, dirname(dirname(abspath(dirname(__file__)))))

try:
    from pyscanfcs.gui_wx.main import Main
except ImportError:
    print(sys.exc_info())
    warnings.warn("Frontend of `pyscanfcs` will not be available." +
                  " Reason: {}.".format(sys.exc_info()[1]))
else:
    Main()



import numpy as np


def reduce_trace(trace, deltat, length):
    """Shorten an array by averaging.

    Given a `trace` of length `len(trace)`, compute a trace of
    length smaller than `length` by averaging.


    Parameters
    ----------
    trace : ndarray, shape (N)
        Input trace that is to be averaged.
    deltat : float
        Time difference between bins in trace.
    length : int
        Maximum length of the new trace.


    Returns
    -------
    newtrace : ndarray, shape (N,2)
        New trace (axis 1) with timepoints (axis 0).

    """
    step = 0
    while len(trace) > length:
        N = len(trace)
        if N % 2 != 0:
            N -= 1
        trace = (trace[0:N:2] + trace[1:N:2]) / 2
        step += 1
    # Return 2d array with times
    T = np.zeros((len(trace), 2))
    T[:, 1] = trace / deltat / 1e3  # in kHz
    T[:, 0] = np.arange(len(trace)) * deltat * 2**step
    return T



import pathlib

import numpy as np

from pyscanfcs import openfile


def test_open_dat():
    here = pathlib.Path(__file__).parent
    f16 = here / "data/n2000_7.0ms_16bit.dat"
    f32 = here / "data/n2000_7.0ms_32bit.dat"
    info16 = openfile.openDAT(str(f16))
    info32 = openfile.openDAT(str(f32))
    
    assert info16["system_clock"] == 60
    assert info32["system_clock"]== 60
    assert np.all(info16["data_stream"] == info32["data_stream"])
    ref = np.array([1, 1, 21420, 21418, 21420])
    assert np.all(info16["data_stream"][-5:] == ref)


if __name__ == "__main__":
    # Run all tests
    loc = locals()
    for key in list(loc.keys()):
        if key.startswith("test_") and hasattr(loc[key], "__call__"):
            loc[key]()



#----------------------------------------------------------------------
# This file was generated by /usr/bin/img2py
#
from wx.lib.embeddedimage import PyEmbeddedImage

Main = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAQAAAAEACAYAAABccqhmAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAAMqAAADKgBt04g1gAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "ACAASURBVHic7Z17fFTVufe/ey6ZSUJCQrjfFBHQgor3HgUvaL3fz2tbz9u3x6PtEW94/Nj3"
    "VD3HxlhPtfb2aq2t1dZe31prraKotaKAWo6igMpFlDsCIQQSICGZzGWdP1ZGwszaM5PJzJ49"
    "M8/385lPcK+dvR8wv1/W5VnPspRSCIJQnngKHYAgCIVDDEAQyhgxAEEoY8QABKGMEQMQhDJG"
    "DEAQyhgxAEEoY8QABKGMEQMQhDJGDEAQyhgxAEEoY8QABKGMEQMQhDJGDEAQyhgxAEEoY8QA"
    "BKGMEQMQhDJGDEAQyhgxAEEoY8QABKGMEQMQygKryRpsNVmBQsfhNiypCiyUClaT5QE+D1wI"
    "TAVG9n5GAMHe29qAZmAHsA1YBMxTjepTxwN2AWIAQlFjNVlB4DK06M8HGrJ81PvAPOBZ1aiW"
    "5Cg81yMGIBQlVpPlBa4GGoFxOX78y8B/qEa1NMfPdR1iAEJRYTVZFvC/gG8DU/L4KgX8CbhL"
    "NaqP8/iegiIGIBQNVpM1Di3Kkx18bQS4D2hUjaUnFjEAoSiwmqxTgGfQE3qFYC7wFdWo9hXo"
    "/XlBDEBwPVaTdTXwKFCR6feM9FhM9Xho8Fg0WBb1lkUPsDumaFWKZqV4PxIl1L9QVgKXqka1"
    "rn/f5l7EAARXYzVZDwD/N5N7j/J6mOHz8g8+L4d50qe49ADLIlEWR6O8Ho6yKzMt7AYuUo1q"
    "cSY3ux0xAMG1WE3WHODBdPdN83qYHahgujf7vLYQ8HRPmN/1RNiXXhM7gBNVo9qS9QtdghiA"
    "4EqsJuss9HKcz+6eoZbFbcEKZvq8OXtvh1I80RPmqZ4IaZSxFJihGlVXzl5eACQVWHAdVpM1"
    "EXiKFOKf5vXwi+pgTsUPMMiyuDlQwT2VAYJWyluPA57I6csLgPQABFfRm6//HjqV18hFfh+3"
    "BSvw5zmWdbEYt3eF2B5LqZFvqEb1gzyHkjekByC4jdmkEP8Vfh+3OyB+gIkeD49WBRlupewK"
    "NFpN1lAHwskLYgCCa7CarGrgTrv2Y71ebglmvBKYE4ZYFvdVBlKtP9YAdzgWUI4RAxDcxL8B"
    "w00NIz0W91ZWkNsRf2ZM8Xr4ZmrjucFqssY4FU8uEQMQXIHVZNWTYr2/MRhgcOqueF451+/j"
    "XL/tnGQQ+JaD4eQMMQDBLVwDDDY1zPB5OWoAa/y54msV/lRzD9cW41xA4f9VBUFzkemiB7gu"
    "4MSUX3pGeSwuq7DtBXjR9QiKCjEAoeD0dv9nmNpm+b1MyCCt1yn+OXUv4ELnIskN7vmXFcqZ"
    "87BJ+pnls/2NWxDqLIvj7JOPzrWaLHcFnAYxAMENXGy66AdO9BZi3j81M+wNoA441cFQBowY"
    "gOAGjjZdPMbrpbJwE/+2nJo6/fhEp+LIBWIAghsYbbrohpl/E8MtixEeW2ca6WQsA8Wd/8JC"
    "2dBb1bfe1DbMXmQFp8E+J6FQFYuyQgxAKDSj7BpSiKzg1NvHJj0AQegHxu4/uNsAhogBCEJO"
    "KLX96EX19xEDEArNNruGVhfXqthtH9sOJ+MYKGIAQqHZbteQYZHOgpDCAJqdjGOgiAEIBUU1"
    "qhC60m4SO1NX4ikou+xjEwMQhH5iHAZ8GI05HUdGNMcULdIDEISc8b7xYjRKpwuHAW9Foqma"
    "33EqjlwgBiC4gedNFyPAOy7sBbwZtTWANuDvDoYyYMQABDfwMhA2NbwejjgcSmp2K8Vy+x7A"
    "y6pRpeweuA0f8C/owoYxIGrziaENubv309Xna+Kf+3ncmlDuqEa1x2qy3gBmJba9HomyNhbj"
    "cJfUBHgiFDY7lWaec5HkBl+fT66IoLtCfT+7e7+KOQh2PI/BABTwaCjM9yoDzkeUwNaY4nn7"
    "HkkYeMnBcHKCD3JeaNUHDOv9JLKfA2bQ1xj2oHsZQvnyBHAXMCSxYXEkyrJolGMLXBvg56Ee"
    "UgxIHlONyric6WYspdQN6KqmhSSGNoG+phD/ur+AcQkOYjVZ3wTuN7UNtywerw6mysHPK/PC"
    "Ee7r7rFr7gImqkZlm9TkViyl1L9xoBfgvjUX6AC29PnsKWw4Qr6wmqwqYB02G2qO8np4qCro"
    "yKlAfVkZjXHT/u5UY/8HVKP6pnMR5Q5LKfVdINFWVZ+vff8cM3wSr+d7FnQvB8xgM9oghBLB"
    "arJuBB62a7/E7+PfHTwdaKdSfL2zO9W+hD3AYcXY/Qc9Xjf1qayEr/0lip4MjCT8ORfj/Fr0"
    "2XHx8+PaOLiHIEOG4ubnwNeA6abGueEI3cA3gxXke1rwo2iMO7tC6TYl3VWs4gfdA3jAwfcp"
    "zMYQJXfDj13onsEW4FP0EqVQRFhN1nh0dmCd3T1TvB7uqwykO7gza14JR7i/uwfbUb/m16pR"
    "XZ2XABzC6RLGFrrYq2kYFzeEvsaQTa+hofdzLNpUdnJwDyHFUE5wCZOAylQ3rInGuLazm1uC"
    "FZzl82bdVU1kj1I8Hgrzl/QJSIuB63L02oJhKaW+jc4I9OLOzMAI0IPOIehhYD2FCLAeWAVs"
    "RJYeXYfVZJ2FzglIaQB9mezxMDvg56TU1XpT0qXgyXCYP/RE2J9+/8EW4ETVqIpq778JSyk1"
    "LeGah4MNIf41AFSglwwDhk8Q/T8t30OzMNoI4p9sDWE/8BGwmiIr4lCqZCP+vkzxejjV5+UU"
    "r5cpXk/aXkGXgnejURZHoiyMRNmT2cajFuA81aiWZROj2zAZwECpQB/yWNv7if+5htwnHYE2"
    "hHjvIEx2hrAL3Sv4CNiXu9CETBmo+BMZYllM9XposCyGeCyGWBY9ShcZ2aUUzbEYK6Ox/o4H"
    "lwOXqka1ORcxuoF8GIDtu4BqDjaF+NeqHL1DkWwI/f3+LWgz+CSL7xeyIEPxL0eX3LatIpxn"
    "/gRcrRpVSa0yOWkAqfCRbAx1pJgFzhDFgaFCN/3LUQgDa9FmsBl3JkkVPRmK/030ybs1wJPA"
    "aQ6EFqcHaFKN6jsOvtMx3GIAdgTQWWGjer/WDvB5YQ7sXuzPBGAHeniwCmgdYAxCL/0Rv2pU"
    "nyV8WU3WxcB/AUflMbwY8DugUTWqjXl8T0FxuwEkUs3BhjCQoUN8C3N/dyjuBFb2fmR3Y5Zk"
    "K/4+3+8BrgLuAQ7LcXjPAv+pGtXKHD/XdRSbASRSw8GGkM2mphgHzKA/Y/4e9Lj0vd7vFTLE"
    "arLOBuaSpfgTnuUHLuz9XECKg0ZSEAOWoPfzP6sa1YdZPKMoKXYDSKSOA2YwEnPCUSoiHBgi"
    "ZDpfEAE+AN5F9iWkJZfiNzzbQieAnQ9MRpvBqN6v9ej/VzvQRUjjn7eBF1Wj2tm/v0lpUGoG"
    "0BcLvbd8JHAIMLSf39/DATPIZAIwih4WLEF2LBrJp/gzeHcACKtG5Wjyl9VkjQGuAf43+pfE"
    "/1ON6l0nY0hFKRtAIrXARPR4sbof36fQY/1M5wti6AnDd7Cpd1+OFFL8hcJqso4DXiX59ONp"
    "bplfKCcD6MsItBkcQv+GCTF0BmEn6XsFCp1L8A46e6xsEfEn8WPVqOY4HJIRpzcDuYUdvZ+3"
    "gXFoMxhN+u3PHmAQugcRNwK7LqWFHodOBjb0vsv2HLxSRcRvxDW/dMvVAOJE0ZuCNqJXECag"
    "zSCpLl0C8azGKtIbAb3PnYDOMnwbnVhU8oj4bXncoXDSUq5DgHTUowV7GJnnGsSNIJPVg+1o"
    "I1ifVXRFgIjfltXAdNWo0pQacAYxgNRY6GWkw4BDyWy7dBfaCDI50aIZeI0iO08uHSJ+Wz4C"
    "zlSNyjX/v8UAMqcKXYZsMpntauxG5wWkMwIFfIgWRNFXLxLx2+I68YMYQDYE0EYwhcxWEEJo"
    "I0iXZdgFLELnEhQlIn5bXCl+EAMYCBXAEcDnev+cjh60EaQb+20F5lNkm45E/La4VvwgBpAL"
    "fOhhwVQyK2YRRhcdSWUEMWAZuu6cKyaLUiHit8XV4gcxgFziQReznEZmmYZdaCNItXzYASwA"
    "Ph5ocPlCxG+L68UPYgD5wINeQjyK9PULFNoE0lWZ2YReLWgbcHQ5RMRvS1GIH8QA8okFjAeO"
    "JvUPC+hhwV5STxRG0ZtJ3iazJca8IuK3pWjED2IATjEWbQTpdiTuR3f7Uw0L9gCvU8AkIhG/"
    "LUUlfhADcJrDgeNJXTo9hh4WpCsysg5tBHtzE1pmWE3WF4DnEPEnUnTiBzGAQhBAF62YnOa+"
    "MPq3farufhhtAityE1pqRPy2rAZmFZv4QQygkAwFTkYfY5aKTvSwINX241Xo3IG8lTEX8dtS"
    "tOIHMYBCE98yfBypswpj6K5+qlTh3cAL5CGBqFf8c0ldc1HEX4SIAbiDIHAC6avb9qCNwG5Y"
    "EEEPCXJW1FLEb0vRix/EANzGcODzpD8QZR96aGDHavQP8ICGBCJ+W1ajJ/yK/kxJMQD34UHv"
    "MTiG1MOCEHqS0G7JsA09JMiq2q2I35aSET+IAbiZKvSw4NAU98SAduz3C0TQqcQf9OfFIn5b"
    "Skr8IAZQDIxCrxakSitONyT4CP3DnXZjkYjflpITP4gBFAseYDqpi0kOeEgg4relJMUPYgDF"
    "xhhgBvaZhFkPCUT8tpSs+EEMoBipQh+PPTzFPemGBGuAv9FrFCJ+W0pa/CAGUKxY6CFBquOx"
    "0w0J2oG5VpN1LCJ+EyUvfsisyq3gPhS6YtCr2GcHBtDpxnblyur+tv5vd48bPO55RPyJlIX4"
    "QQyg2NkGPI99WXEP+pCTpApFCzYumPzKuleuu+KIKwKThkyye76Iv8SRIUBpYKHrDRyT4p7P"
    "hgQLNi6YPO+TedfGVMwHoJRi/ob5fLDjoLlBEX8ZIAZQWoxCrxLY7diLvbn5zaHPrXnun+Pi"
    "78viLYtZ/OliEPGXDWIApUcQvUowMrFhefPyCa+uf/Xy3V27vaGo+aTzDW0bVizctPCUXf++"
    "a1+e43QEEX9qxABKEwu9QjA9fmF58/IJ8zfMvzymYl6A9u52uiMHzx/WB+s3zDl5zmO1gdoV"
    "wItkds6haxHxp0cMoLQZAcxc3rx8al/xx9kb2sv+sC5I3Ef88a7BFnQBENefS2BCxJ8ZYgAl"
    "zl2v3XVGVEV/VF9ZbzwKvrOnE5/Hlyj+ODuBZ0idVOQ6RPyZIwZQwtzy8i2nvL7h9Ye8Hm/g"
    "zEPPZFTNqKR76oP1W6+adtVvqyuq7fYI7AH+jE4ccj0i/v4heQAlSlz8ChWIxCLM3zCfDe0b"
    "DrqnPli/9Z+O+qenqyuqvWjBWIZHDQa+TOrUY1eQofhXIeL/DDGAEqSv+OPXYirGG5veYPXO"
    "1cAB8Vf5q+Jj/AA6acj0M1EFfBF90Ikr6Yf4Z4n4DyAGUGKYxN+XJduWsL5t/ZoE8cfxY28C"
    "FcDlpC9n7jgi/uwRAygh0okfoNpfvXT2CbO/WuWvWmBziw9dqtw0aegFLkRnHboCEf/AEAMo"
    "ETIV/6MXP3r9MSOO2Q+sRVcQNq31e9E9AVNNQgs4G5iSg7AHhIh/4IgBlABZiD/OFnRdAFP1"
    "4PhGIrvdhOdRwDkBEX9uEAMocgYg/jgtwMuYjyi30CZg2i7sBS5FJxs5iog/d4gBFDE5EH+c"
    "NrQJ2B00WofZBPzAFaQ//jxniPhzixhAkZJD8cfpAF7C/mixOszDgUq0CSTVHMg1Iv7cIwZQ"
    "hORB/HFCwCvAdpv2eswTg4PRJpDq2PMBIeLPD2IARUYexR8ngj5peJOhzUIL0LREOAw9J+A1"
    "tA0IEX/+EAMoIua8NOfUPIs/Tgx4A9hqaPOghWj62RkLXIA5pTgrJL03v4gBFAlzXppz6oKN"
    "Cx50QPxxYugzBExzAvE8AdPPzyTgrBy8v7/ib8nFO8sNMYAioADijxNFDwf2GNp82G8gOho4"
    "ZSAvFvE7gxiAyymg+OOE0EI0PduP/VHmnyd1kVJbRPzOIQbgYlwg/jid9DlJKIEAehXAxCz6"
    "uXlIxO8sYgAuxUXij7MHLcyIoa0SqDFct4DzgXGZvEDE7zxiAC7EheKP04qeGFSmkNB1AxKJ"
    "pwynLCgi4i8MYgAuw8Xij7MNfW6AiVrMKcMVaBMwHkEm4i8cYgAuogjEH2cDsMSmrQ5zRmAN"
    "egfhQYj4C4sYgEsoIvHHWQ18aNNWhzll+DDg+Ph/iPgLjxiACyhC8cdZBnxsuB5PGTb9fM0E"
    "Ron43YEYQIEpYvHHeRvYbLjuwbw86Hlr81s3BH3B+aQW/0pE/HlHDKCAlID4Qa8ILMJ8RHmA"
    "hG3CS7cvHTv347m3nTvxXLsEItDinyXizz9iAAWiRMQfJwYsxJwtWENvHYGl25eO/ePKP86O"
    "xCKVE4dM5LhRx5meJeJ3EDGAAlBi4o8TQvcETNQta142Pi7++MWZ42cyctBBhxiL+B1GDMBh"
    "SlT8cVrQE4MHsXb32lF/XffX6/qKH8Dr8XLhpAsJeAMg4i8IYgAO0rSwaVoJiz/Oh+hkIQDW"
    "7l474vmPn/+S1/IGqyuSq4YNDg7msiMu24yIvyCIATjI/PXz/6XExR/nDWB/XPyRWCQAUFNR"
    "Q4X34LKCgyoGNf/ovB/9QjWq0YUItNwxHhkt5J75G+bXtXe32xbKKCHxA4R+/+Hvm1s6Wr4Z"
    "UZGDDK8uWEfr/lZiKsagikHNs0+Y/dNRg0Z1AKejew7SC3AQ6QE4xCljT9nr8/iMJatKTPzc"
    "u+jeI7/31ve+u2TbkqTcf4/lYXBwcKL4QW8augj7g0iEPCAG4BCV/srY2Nqxf068Xorif3rV"
    "049HVbT2w5YP2bovuaxgQ2VD600n3fTrPuKPUwec40igAgCWUmpaoYMoF3Z37fZ+45VvfGHl"
    "zpVfsSwrPGnIpL/cc8Y9f51QPyFU6NhyQV/xx68FvAEunnIxVX69U3hQxaDWL0370h+HBIfs"
    "B3ZjLjLyKvCBI0GXOWIAJcyKlhWVP1nyk5N3dOw4ZH94/9DuSPfQUDQ0NBKL1Hssz/4Kb0Vr"
    "wBdorfJV7aoN1DafM/GcJV895qvb0j85GZP44wyvHs65E8+lJlDTV/ygE4hae7/2JQL8Dm0Q"
    "Qh4RAygxfvP+b0a/8PELp3+679PTOkIdJylUv8bUAV9g7bCqYQunj5y+6I4ZdyyvDdQmijOJ"
    "VOKPc/LYkz994AsPPNdH/HFC6KPJEtkC/Kk/sQv9RwygRLhj/h3Hvb7h9Tmd4c7j09+dGT6P"
    "r3Vi/cTHHzr/oadG14w2nSCckfgD3sDab8/69rXnH37+scAYwy370HUHE3kR+Cir4IWMEAMo"
    "cu5/8/4pz3/8/C17Q3tn5usdfo9/2+eGfe6RRy585Pm+PYJMxd90ZtO1F066cDd6c9DFmEuH"
    "tZJcb7ATeALzPIGQA8QAipQVLSsqb37p5v9o3d96CTk8iScVAW9g/SVTLrnzW6d/a2UW4o8z"
    "HDiX5Jh7MI/5l6LrEAp5QAygCPnlsl+O+em7P32oO9Ldr5LbucDCCh1ad+hjm/ds/moW4o9z"
    "NDDdcL0d6E64ptATgjuzDlqwRQygyLjtldtOnr9+/vejKppqP/1BWMqDr2cIvnAd3p56/JFa"
    "op5uIhXtRP1thCt2EfPmbiUyjfhB559cTHLBkBha6IlVh7cBT+YsQOEzJBW4iLjyT1de9VHr"
    "R7eTQQKXJ1rFoPZjGNR2PNXtR+ONJG/EiaOsGN01n7Cvbhmd9csIVX6adYwBb2DtvbPuvea8"
    "w88zzezHiQH/jR4KHBQ2MAg9KdiX0cBU9I5BIYdID6BIuPrZqy94b/t73013nzdcQ8PWy6hv"
    "ORsrlp2/7x+8mp1jn6KrxlTuz54Mxd+XGehCoYmYJgS70BOCiUMEYQCIARQB33njO0c+ufLJ"
    "3yplv5PQEw3SsP0i6refjydqLL/fbzrqlrNz/FOEqjalvTcL8YM+J+BykisIh4Fdhvs/QGcJ"
    "CjlC9gK4nJfXvlz/9KqnH0wl/oru4Ryy8h4aPr08Z+IHGNQ+nUNX3MPgnaelvXdyw+Tf9lP8"
    "oH+bJxUQQRtCpeH6UcCIfr5DSIEYgItp6Wzx3b3g7h+FY+FRdvdU7T2S8SvvIbDflF8zcKyY"
    "j1HrrmP45qtA2f+4rNi54o773rzviCxesQbz8l8NyT+fFnA2Di17lgNiAC7mhnk3fClVZt+g"
    "thMYt/p2fGHTuZy5Zci2ixi97kbstKeUCj770bP/mcWjFXpCMJH4hGAiI9A9ASEHiAG4lPd3"
    "vF+1tm3tv9q1BzoPYfTaG7CUcws5ta2fZ9iWL9q27w/vP+b6edefkcWjWzEfMFKF+YShGZiH"
    "CEI/EQNwKXe9dtf/icaiQ0xtvnAd49bchidqOy2QNxq2XkJt6wzb9ne3vTunK9yVzc/VMvTG"
    "oERMyUZBIP3EhJAWMQAXsmDjgsGb9my62q599Nob8fU0OBjRwYxa/3Uqus3TEt2R7kmz580+"
    "P4vHhoD3DNf9mPcOTEXnBwgDQAzAhfxw8Q+/FFMx0/iXQe3TqdrzOadDOggr5mPolitt21e0"
    "rLg+y0evxZzya5oQBDgLmRAcEGIALqS5o3mWucVKKTwnqd11EoHOQ4xtPdGeQ364+IeHZ/no"
    "/yY5FdhCm0Aiw4ApWb5HQAzAdTy35rmhXZEu46/4mt0nEew81OGI7LBSTggu2rQo2zF6G/ro"
    "8UQqMRcMPSnL9wiIAbiOp1Y+NRObbm1dyxnOBpOGQe3H4OsxH/Db3NF8+gAe/T725wwmMhSY"
    "OIB3lTViAC5j857NRuF4ogGq9hZ27J+MxaB2065e6Ax3Tl+wcYHpePBMCAPvGq77MfcCTs7y"
    "PWWPGIDL2Bfad4LpetWeo7Pe3JNPBrUZT/gF8Pxx5R+PHcCjN2KeEDRNjo4EzBMSQkrEAFzE"
    "5j2bK6IqavytWb3XnXu2qvZOtW1r62obNsDHm0qDV2BODpJeQBaIAbiIRZsW2S7u+0OFW/dP"
    "hScawBM11xroDHcONOitmPcJmHoBY5G8gH4jBuAi1u5eO9SuzduTcQEgx/H3mIf6XeEu279P"
    "P/jQcC2AuZiN9AL6iRiAi2juaLYVjD/sXgPwhs0GEIqGcmEAm4A9huumXsAEdNFRIUPEAFxE"
    "T7THNrnfijmf958pnphpSA5RlbPNCqZeQBDpBQwYMQAX0VDVYKqCA0Ckot3JUPpF2L/XeD3g"
    "Ddj+ffrJBpLrBAKYJh8mAcZNVEIyYgAuYnzt+Fa7tqi/v8V2nCPqN/XQIegL5soAFLDCcL0S"
    "fax4IpIdmCFiAC7iuNHH2RpAxEZkhUcRtekBVPmrbP8+WbAOc3agqRdwBMklxwUDYgAu4tRx"
    "p+6zsIzHYHVXb3Q4mswIVW1FWVFjW22gNpcGEMNcFryK5J9jD3BiDt9dsogBuIyAL7DedL2z"
    "zlQ7s/CkimtKwxTj32UAfIy5LLipFzAV80qB0AcxAJcxrGrYItP1UOU2eoI7nA4nLfvqzQbg"
    "8/h23D7j9jU5fl0UWGW4buoFeAFjWrVwADEAl3H86OMX2rV1DHnHyVDSEqloo3vQJ8a2IZVD"
    "jEaWA9aQfFqwhbkXcDRSOzAlYgAu484Zd67werzGKf9do14k5nXPwTi7xjyHsmLGtikNU2yN"
    "bICEMdcLqCJ5G7UPOCZPcZQEYgAuo9JfGasP1ht/e0b9e2kb+bLTIRkJB1toH/6asc2yrNCt"
    "/3CrqdR3rvgIbQQHvRb7uQDBBjEAF3LymJOfsWvbNXoeUV+nk+EYaR3zjO3s//Cq4c9OGjIp"
    "d8cNJxNCDwUSMRUPHYxsErJFDMCF3H/2/UtrAjVvmtpi3v3smPCE0yEdREfdcvYMfcvYZllW"
    "6IYTb3jUgTBWoScF++JBbxRKxG2VVFyDGIBLuWjSRQ+RXBwTgL0Ni9k19i8OR6QJVX3KtkkP"
    "g83Yf0zNmN9fceQVpkIeuaYbvVEoEdOk32TMGYNljxiAS7lz5p2r64P1r9i17xz7Z/Y1LHEy"
    "JCL+fXw6+QfEvF3Gdo/l2XfHjDt+4WBI6wzXAiRPBgYxH0Ne9ogBuJirjrrqxx7LY1Ybiu0T"
    "H2Hv0HzOtR0gEtjFliPuIxxssb3nsPrDHjvtkNPMecH5oZnk9GALLfhEZBhgQAzAxVx/wvWb"
    "Thpz0p3YDAVinh62Hf4wreP+bHdLTuiq+YSN075FqNrU49bUBevmP3XlU7/KWxBmFGDKNjQN"
    "AyZgNoayRgzA5Tx28WOvjh88/uf2dyhaxzzD1sk/zvmGIWXFaB8xny1H/hcRv/125KAv+MnD"
    "Fzx8p9/jz58L2WMygAqSx/we9CYhoQ+WUsqd1SaFzwjHwtaZvzrzwT2hPWemus8TDVDffC5D"
    "tl+EN2Ku05cZir0N77Br7NOEKrelvNNredvnnDzny9cce83WAbxwoFxEcg2AfUDiemkz8P8d"
    "iahIEAMoEpZuX1o9+4XZT3RFuo5Md68nWkV989kMajuByo7DyPT4vEhFGx11y2kb8SqhDHYf"
    "WpbVfc5h59z4/XO+X+gc5SNJ3v0XQR87nsgT6NOHBMQAiopPdn8SuHbutfe2dbWdl+n3eMOD"
    "GdQ2nUDXOHzhwXjDg/GGa1DebiL+vUT8e4hUtNJZ92HvluPMevF+j3/7lVOvnHPHjDs+yvKv"
    "k0uCwJUkO90ukjMG3wbMSQxliBhAEXLJk5d8fUPbhpsp0Mm41f7qpXedftetF0660FSyu1Cc"
    "BYxJuLYfSFyV2As87khERYBMAhYhc78897EZ42fM8Vgex3OCh1UPe3ruVXOvdZn4wZwTYJr1"
    "r0WfISAgPYCi5pnVzwx7ZMkj17V0tvyjQuX13LAqf9UHM8bPeOgH5/zg7Xy+ZwB4gS+SfGpQ"
    "G3rvQF9WALZJVuWEGEAJ8Iulvxj7mw9+c9Purt3nk+NeXcAbWHvC6BMe+tlFP3s9l8/NE6cA"
    "hydc6wYS1zBDwKPoicKyRgyghPje37836W/r/nb5rq5dp/dEe8Zn+xyP5emsDdT+fdKQSa/8"
    "5IKfvFLprzQn/ruPEcC5hust6JqCfXkBXWKsrBEDKFEefufhQ1/b8Nrp2zu21+4qzQAABVJJ"
    "REFUz+yOdB8ajUUbbIYJymt59/i9/h31wfp3pg2ftvD2Gbe/N7x6eLH+dvxHkusC7CU5ZXg9"
    "8KwjEbkYMYAyIRwLW4s2LRr8fvP7Q7d3bK+vqajZP7lhcuusCbN2FbHYTRwLHJVwLYxeEuxL"
    "DPgZ5iKjZYMYgFBq1AKXGa7vJLl+wDzMhUXKBlkGFEqNvZgzAE1LguPyHIvrEQMQShG7DUKJ"
    "iAEUOgBByANbDNdMBlAP1OQ5FlcjBiCUIp1AR8I1i+QkISjzXoAYgFCqbDdcM/UCss6XKAXE"
    "AIRSpdlwzVQxWHoAglCCmAzAT/IOyhqgLv/huBMxAKFU6QISa6TJPEACYgBCKWPqBcg8QB/E"
    "AIRSRuYB0iAGIJQymc4DVAEN+Q/HfYgBCKVMCHMBUBkG9CIGIJQ6mc4DlOUwQAxAKHUyTQga"
    "S4GKrBYSMQCh1DEdZmiaBwgCw/IfjrsQAxBKnR7M24NlHgAxAKE8kOVAG8QAhHIg04nAspsH"
    "EAMQyoEWks8885H88+8HBjsSkUsQAxDKAbuDQk1VkhNPGS5pxACEcsF0lJnJAOrzHYibEAMQ"
    "yoXEnYEgPQAxAKFsSDwlGPR5golID0AQShDpARjwHfmTI/+1LljXMqJ6xM6JQya2nH7I6S2X"
    "TLnE9I8lCMVMJ/pgkL6/9b3oZb++KwRV6ByBxBOFSxKLu5OWR7Asq8fv8e8MeAMtAV9gZ01F"
    "zdYTxpzw5sPnP/ze8OrhiaerCEKxcDHJXfxWkk8J/gPmPQQlh9EA7PB6vHuHVg5dNHX41Nfv"
    "PuPut2aOn9mZz+AEIcecDhyScK2d5PMBXwZWORJRgemXARz0jZYVHlE94sU7Z9758M0n3WzK"
    "tBIEtzEdODrhWgfJZwi8DbzlSEQFJutJQKWUv7mj+dJbXr7lhQkPTrht3ifzanMZmCDkgUxX"
    "AspmInDAqwBKqcDG9o1XX/qHS1+e+sjUa9buXmvKsRYEN2AygLJeCcjZMmBURWtW7Vx16/RH"
    "p//6saWPDc/VcwUhh2S6FFhHmWwKynkeQGdP57Qb5934h9kvzJ6a62cLwgAJo88L6ItFsg68"
    "QFkMafOSCBSOhYc/tvSxX5/xqzPOz8fzBWEAyDCgD3nLBIypWGDhpoUPTH1k6jX5eocgZIFk"
    "BPYh76nAq3auulV6AoKLkD0BfXBkL8Abm9/4tswJCC4h0yGAGECuiKlY4JfLfvmQrA4ILkCG"
    "AH1wbDdgOBYefutfb31Q8gSEAtMBxBKumYYA1ZiNoaRwdDtwZ0/ntEufvPQrTr5TEBJQJKf+"
    "gtkEqvIcS8FxvB7AmtY1X5O0YaHAJG7+AbMWgvkOpNA4bgBRFa256cWbvu70ewWhD6a9/iYt"
    "VOY7kEJTkIpAm/ZsuurH7/x4ZCHeLQhk3gMQA8gHSqnAd974zk2FeLcgYO4BmHL/ZQiQL3Z0"
    "7rjgjc1vVBfq/UJZIz2AXgpmAEop/90L7j61UO8XypoewzUxAKdZ2bLyzEK+XyhbZBWgl4Ia"
    "QGtX62ktnS2m9VdByCeyCtBLQQ0gGovW3vTSTccXMgahLJE5gF4KfjDIu1vfnVHoGISywzQH"
    "YFoFKPm09YIbwL6efWMKHYNQdmRaCbvkh6cFN4BQJDSs0DEIZYfpcBtTD0AMIN+EoiHZIiw4"
    "TeJuQBADKAzhWFh6AILTyBCgl4IbgFKqYu6auYMLHYdQdiQOA0w9gILrI9+44i+4cNNCGQYI"
    "TmMaBiTiocTPB3CFAazbvU4MQHAamQjEJQawo3OHzAMITpNJDwBK3ABcUfOsvbtdegCC09j1"
    "AFRvW/xT0kOA/wFd9Rqp2V9jXwAAAABJRU5ErkJggg==")
getMainData = Main.GetData
getMainImage = Main.GetImage
getMainBitmap = Main.GetBitmap
getMainIcon = Main.GetIcon



"""Documentation-related methods"""
import os
import platform
import sys

import astropy
import matplotlib
import multipletau
import numpy
import scipy
import skimage
import wx


from .._version import version as __version__

def GetLocationOfFile(filename):
    dirname = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    locations = [
        dirname + "/../",
        dirname + "/../pyscanfcs_doc/",
        dirname + "/../doc/",
    ]

    for i in range(len(locations)):
        # check /usr/lib64/32 -> /usr/lib
        for larch in ["lib32", "lib64"]:
            if dirname.count(larch):
                locations.append(locations[i].replace(larch, "lib", 1))

    # freezed binaries:
    if hasattr(sys, 'frozen'):
        try:
            adir = sys._MEIPASS + "/doc/"
        except:
            adir = "./"
        locations.append(os.path.realpath(adir))
    for loc in locations:
        thechl = os.path.join(loc, filename)
        if os.path.exists(thechl):
            return thechl
            break
    # if this does not work:
    return None


def GetLocationOfChangeLog(filename="CHANGELOG"):
    return GetLocationOfFile(filename)


def GetLocationOfDocumentation(filename="PyScanFCS_doc.pdf"):
    """ Returns the location of the documentation if there is any."""
    return GetLocationOfFile(filename)


def description():
    return u"""PyScanFCS is a data displaying and processing
tool for perpendicular line scanning FCS utilizing 
correlator.com correlators. PyScanFCS is written in Python."""


def info(version):
    """ Returns a little info about our program and what it can do.
    """
    textwin = u"""
    Copyright 2011-2012 Paul Müller, Biotec - TU Dresden

    Data processing for perpendicular line scanning FCS.
    """
    textlin = u"""
    © 2011-2012 Paul Müller, Biotec - TU Dresden

    Data processing for perpendicular line scanning FCS.
    """
    if (platform.system() != 'Linux'):
        texta = textwin
    else:
        texta = textlin
    one = "    PyScanFCS version " + version + "\n\n"
    lizenz = ""
    for line in licence().splitlines():
        lizenz += "    " + line + "\n"
    ret = one + lizenz + texta
    return ret


def licence():
    return u"""PyScanFCS is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published 
by the Free Software Foundation, either version 2 of the License, 
or (at your option) any later version.

PyScanFCS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License 
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


def SoftwareUsed():
    """ Return some Information about the software used for this program """
    text = "Python " + sys.version +\
           "\n\nModules:" +\
           "\n - Astropy " + astropy.__version__ +\
           "\n - matplotlib " + matplotlib.__version__ +\
           "\n - multipletau " + multipletau.__version__ +\
           "\n - NumPy " + numpy.__version__ +\
           "\n - scikit-image " + skimage.__version__ +\
           "\n - SciPy " + scipy.__version__ +\
           "\n - wxPython " + wx.__version__
    if hasattr(sys, 'frozen'):
        pyinst = "\n\nThis executable has been created using PyInstaller."
        text = text + pyinst
    return text


support = """
PyScanFCS has no funding and no developer community. My personal
objective is to keep PyScanFCS operational on Linux and Windows which
is currently limited by the free time I have available.

An active community is very important for an open source project such
as PyScanFCS. You can help this community grow (and thus help improve
PyScanFCS) in numerous ways:

1. \tTell your colleagues and peers about PyScanFCS. One of them might
   \tbe able to contribute to the project.

2. \tIf you need a new feature in PyScanFCS, publicly announce a bounty
   \tfor its implementation.

3. \tIf your research heavily relies on FCS, please consider diverting
   \tsome of your resources to the development of PyScanFCS.

4. \tYou don't have to be a Python programmer to contribute. If you are
   \tfamiliar with reStrucuredText or LaTeX, you might be able to help
   \tout with the online documentation.

5. \tPlease cite: Müller et al. Methods in Molecular Biology, 1076,
   \t(635–51), 2014.

If you are planning to contribute to PyScanFCS, please contact me via
the PyScanFCS issue page on GitHub such that we may coordinate a pull
request.


Thank you!

Paul Müller (May 2018)
"""


# Standard homepage
HomePage = "http://pyscanfcs.craban.de/"

# Changelog filename
ChangeLog = "CHANGELOG"
StaticChangeLog = GetLocationOfChangeLog(ChangeLog)


# Github homepage
GitChLog = "https://raw.github.com/FCS-analysis/PyScanFCS/master/CHANGELOG"
GitHome = "https://github.com/FCS-analysis/PyScanFCS"
GitWiki = "https://github.com/FCS-analysis/PyScanFCS/wiki"
